@@ -5,12 +5,14 @@
     clippy::cargo,
 )]
 
-#![allow(clippy::module_name_repetitions)]
 #![allow(clippy::cargo_common_metadata)]
-#![allow(clippy::missing_docs_in_private_items)]
+#![allow(clippy::module_name_repetitions)]
 #![allow(clippy::missing_const_for_fn)]
 #![warn(clippy::clone_on_ref_ptr)]
 
+#![warn(missing_docs)]
+
+mod reference;
 mod ui;
 pub mod layout;
 pub mod widget;
@@ -19,4 +21,5 @@ pub mod unit;
 pub mod event;
 pub mod plugin;
 
+pub use self::reference::*;
 pub use self::ui::*;
\ No newline at end of file
