@@ -12,10 +12,10 @@ use parking_lot::Mutex;
 use crate::{
     context::{ListenerID, WidgetContext},
     event::WidgetEvent,
-    layout::LayoutRef,
     plugin::WidgetPlugin,
     unit::{Key, Rect},
     widget::{Widget, WidgetId, WidgetRef},
+    Ref,
 };
 
 mod cache;
@@ -33,14 +33,15 @@ impl Extend<WidgetEvent> for VoidEvents {
     fn extend<T: IntoIterator<Item = WidgetEvent>>(&mut self, _: T) {}
 }
 
-pub struct WidgetManager {
+/// Handles the entirety of the widget lifecycle.
+pub struct WidgetManager<'ui> {
     plugins: BTreeMap<TypeId, Box<dyn WidgetPlugin>>,
 
     widgets: Arena<WidgetNode>,
     tree: Tree<WidgetId>,
     cache: LayoutCache<WidgetId>,
 
-    context: WidgetContext,
+    context: WidgetContext<'ui>,
 
     changed: Arc<Mutex<HashSet<ListenerID>>>,
 
@@ -59,7 +60,7 @@ pub struct WidgetManager {
     changes: usize,
 }
 
-impl Default for WidgetManager {
+impl<'ui> Default for WidgetManager<'ui> {
     fn default() -> Self {
         let changed = Arc::new(Mutex::new(HashSet::new()));
 
@@ -70,13 +71,7 @@ impl Default for WidgetManager {
             tree: Tree::default(),
             cache: LayoutCache::default(),
 
-            context: {
-                let changed = Arc::clone(&changed);
-
-                WidgetContext::new(Arc::new(move |listener_ids| {
-                    changed.lock().extend(listener_ids);
-                }))
-            },
+            context: WidgetContext::new(Arc::clone(&changed)),
 
             changed,
 
@@ -97,12 +92,14 @@ impl Default for WidgetManager {
     }
 }
 
-impl WidgetManager {
+impl<'ui> WidgetManager<'ui> {
+    /// Create a new `WidgetManager`.
     #[must_use]
     pub fn new() -> Self {
         Self::default()
     }
 
+    /// Initialize a plugin with its `Default::default()` method.
     pub fn init_plugin<P>(&mut self)
     where
         P: WidgetPlugin + Default,
@@ -110,6 +107,7 @@ impl WidgetManager {
         self.add_plugin(P::default());
     }
 
+    /// Load a plugin.
     pub fn add_plugin<P>(&mut self, plugin: P)
     where
         P: WidgetPlugin,
@@ -118,10 +116,11 @@ impl WidgetManager {
         let plugin = Box::new(plugin);
 
         self.plugins.insert(plugin_id, plugin);
-        
+
         self.update_plugin(plugin_id);
     }
 
+    /// Fetch a loaded plugin. Will be `None` if it does not exist.
     pub fn get_plugin<P>(&self) -> Option<&P>
     where
         P: WidgetPlugin,
@@ -131,56 +130,62 @@ impl WidgetManager {
             .and_then(|plugin| plugin.downcast_ref())
     }
 
-    pub fn update_plugin(&mut self, plugin_id: TypeId) {
-        let plugin = self
-            .plugins
-            .get(&plugin_id)
-            .expect("cannot update a plugin that does not exist");
-
-        *self.context.current_id.lock() = Some(ListenerID::Plugin(plugin_id));
-
-        plugin.on_update(self, &self.context);
-
-        *self.context.current_id.lock() = None;
-    }
-
+    /// Returns the widget tree.
     pub fn get_tree(&self) -> &Tree<WidgetId> {
         &self.tree
     }
 
+    /// Check if a widget exists in the tree.
     pub fn contains(&self, widget_id: &WidgetId) -> bool {
         self.widgets.contains(widget_id.id())
     }
 
+    /// Fetch the tree representation of a widget. Will be `None` if it doesn't exist.
     pub fn try_get_node(&self, widget_id: WidgetId) -> Option<&WidgetNode> {
         self.widgets.get(widget_id.id())
     }
 
+    /// Fetch the tree representation of a widget.
+    /// 
+    /// # Panics
+    /// 
+    /// Will panic if the widget is not found.
     pub fn get_node(&self, widget_id: &WidgetId) -> &WidgetNode {
         self.widgets
             .get(widget_id.id())
             .expect("widget does not exist")
     }
 
+    /// Fetch a widget from the tree. Will be `None` if it doesn't exist.
     pub fn try_get(&self, widget_id: &WidgetId) -> Option<WidgetRef> {
         self.widgets
             .get(widget_id.id())
             .map(|node| WidgetRef::clone(&node.widget))
     }
 
+    /// Fetch a widget from the tree.
+    /// 
+    /// # Panics
+    /// 
+    /// This will panic if the widget is not found.
     pub fn get(&self, widget_id: &WidgetId) -> WidgetRef {
         self.try_get(widget_id).expect("widget does not exist")
     }
 
+    /// Fetch a widget as the specified type. If it doesn't exist, or it is not the requested type, this
+    /// will return `None`.
     pub fn try_get_as<W>(&self, widget_id: &WidgetId) -> Option<Rc<W>>
     where
         W: Widget,
     {
-        let v = self.try_get(widget_id)?;
-
-        v.try_downcast_ref()
+        self.try_get(widget_id)?.try_downcast_ref()
     }
 
+    /// Fetch a widget as the specified type.
+    /// 
+    /// # Panics
+    /// 
+    /// If the widget is not the requested type, it will panic.
     pub fn get_as<W>(&self, widget_id: &WidgetId) -> Rc<W>
     where
         W: Widget,
@@ -188,11 +193,13 @@ impl WidgetManager {
         self.get(widget_id).downcast_ref()
     }
 
+    /// Get the visual `Rect` of a widget.
     pub fn get_rect(&self, widget_id: &WidgetId) -> Option<&Rect> {
         self.cache.get_rect(widget_id)
     }
 
-    pub const fn get_context(&self) -> &WidgetContext {
+    /// Get the widget build context.
+    pub const fn get_context(&self) -> &WidgetContext<'ui> {
         &self.context
     }
 
@@ -217,6 +224,10 @@ impl WidgetManager {
         self.modifications.push(Modify::Destroy(widget_id));
     }
 
+    /// Update the UI tree.
+    /// 
+    /// This processes any pending additions, removals, and updates. The `events` parameter is a list of all
+    /// changes that occured during the process, in order.
     pub fn update<E>(&mut self, events: &mut E)
     where
         E: Extend<WidgetEvent>,
@@ -252,9 +263,7 @@ impl WidgetManager {
                     }
 
                     ListenerID::Computed(widget_id, computed_id) => {
-                        let changed = self.context.did_computed_change(&widget_id, computed_id);
-
-                        if changed {
+                        if self.context.call_computed_func(&widget_id, computed_id) {
                             dirty_widgets.insert(widget_id);
                         }
                     }
@@ -344,16 +353,7 @@ impl WidgetManager {
         let mut root_changed = false;
 
         if let Some(widget_id) = self.tree.get_root() {
-            let layout = match self.context.get_layout(&widget_id) {
-                LayoutRef::None => None,
-                LayoutRef::Owned(layout) => Some(Rc::clone(&layout)),
-                LayoutRef::Borrowed(weak) => Some(
-                    weak.upgrade()
-                        .expect("root node should not have been dropped"),
-                ),
-            };
-
-            if let Some(layout) = layout {
+            if let Some(layout) = self.context.get_layout(&widget_id).try_get() {
                 if let Units::Pixels(px) = layout.position.get_left() {
                     if (self.cache.posx(widget_id) - px).abs() > f32::EPSILON {
                         root_changed = true;
@@ -452,6 +452,19 @@ impl WidgetManager {
         }
     }
 
+    fn update_plugin(&mut self, plugin_id: TypeId) {
+        let plugin = self
+            .plugins
+            .get(&plugin_id)
+            .expect("cannot update a plugin that does not exist");
+
+        *self.context.current_id.lock() = Some(ListenerID::Plugin(plugin_id));
+
+        plugin.on_update(self, &self.context);
+
+        *self.context.current_id.lock() = None;
+    }
+
     fn process_spawn<E>(
         &mut self,
         events: &mut E,
@@ -483,7 +496,7 @@ impl WidgetManager {
             self.widgets.insert(WidgetNode {
                 widget,
                 layout_type,
-                layout: LayoutRef::None,
+                layout: Ref::None,
             }),
             parent_id.map_or(0, |node| node.depth() + 1),
         );
@@ -584,7 +597,7 @@ mod tests {
         context::WidgetContext,
         ui::VoidEvents,
         unit::LayoutType,
-        widget::{BuildResult, Widget, WidgetImpl, WidgetLayout, WidgetRef, WidgetType},
+        widget::{BuildResult, Widget, WidgetBuilder, WidgetLayout, WidgetRef, WidgetType},
     };
 
     use super::WidgetManager;
@@ -617,7 +630,7 @@ mod tests {
         }
     }
 
-    impl WidgetImpl for TestWidget {
+    impl WidgetBuilder for TestWidget {
         fn build(&self, ctx: &WidgetContext) -> BuildResult {
             let computes = Arc::clone(&self.computes);
 
